;;;; uuid-eggdoc.scm

(use eggdoc)

(define license
"Copyright (c) 2006, Kon Lovett.  All rights reserved.

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the Software),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED ASIS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.")

(define doc `(
  (eggdoc:begin
    (name "uuid-lib")
    (description (p "OSF DCE 1.1 UUID"))
    (author (url "mailto:klovett@pacbell.net" "Kon Lovett"))
    (history
      (version "1.2" "Port to Chicken 4")
      (version "1.1" "Bug fix for import [Reported by arto]")
      (version "1.0" "Initial release"))
    (usage)
    (download "uuid-lib.egg")

    (documentation

      (p "Wrapper for " (code "uuid/uuid.h") " procedures on MacOS X, "
      "and Linux.")

      (subsection "Predicates"

        (procedure "(uuid? UUID)"
          (p "Is " (tt "UUID") " a uuid?"))

        (procedure "(uuid-null? UUID)"
          (p "Is " (tt "UUID") " the \"null\" uuid?"))

        (procedure "(uuid=? UUID1 UUID2)"
          (p "Are " (tt "UUID1") " and " (tt "UUID2") " equal?"))

        (procedure "(uuid<>? UUID1 UUID2)"
          (p "Are " (tt "UUID1") " and " (tt "UUID2") " not equal?"))

        (procedure "(uuid<? UUID1 UUID2)"
          (p "Does " (tt "UUID1") " order below " (tt "UUID2") "?"))

        (procedure "(uuid>? UUID1 UUID2)"
          (p "Does " (tt "UUID1") " order above " (tt "UUID2") "?"))

        (procedure "(uuid<=? UUID1 UUID2)"
          (p "Does " (tt "UUID1") " order below or the same as " (tt
          "UUID2") "?"))

        (procedure "(uuid>=? UUID1 UUID2)"
          (p "Does " (tt "UUID1") " order above or the same as " (tt
          "UUID2") "?"))
      )

      (subsection "Operations"

        (procedure "(uuid-clear! UUID)"
          (p "Returns the " (tt "UUID") " set to the null uuid."))

        (procedure "(uuid-copy UUID)"
          (p "Returns a copy of the " (tt "UUID") "."))

        (procedure "(uuid-generate [#:method #f])"
          (p "Returns a new uuid.")

          (p "Method is " (code "#f") ", " (code "'random") ", or "
          (code "'time") "."))

        (procedure "(uuid-parse UUID-TEXT)"
          (p "Returns a new uuid from the UUID external string representation "
          (tt "UUID-TEXT") "."))

        (procedure "(uuid-unparse UUID [#:case #f])"
          (p "Returns the external string representation of " (tt "UUID") ".")

          (p "Case is " (code "#f") ", " (code "'upper") ", or " (code "'lower") "."))
      )

      (subsection "Errors"

        (p "Argument errors are signalled using " (code "error") ".")

        (p "When a non-uuid is given as an argument the result is "
        (code "#f") ".")
      )
    )

    (section "License" (pre ,license))
  )
))

(eggdoc->html doc)
